@@ -91,6 +91,10 @@
 #define GPM_SEARCH_HIGHLIGHTING_DIR	GPM_PREFS_DIR "/editor/search_highlighting"
 #define GPM_SEARCH_HIGHLIGHTING_ENABLE	GPM_SEARCH_HIGHLIGHTING_DIR "/enable"
 
+// FIXME: Still have to add schemas
+#define GPM_SOURCE_STYLE_DIR		GPM_PREFS_DIR "/editor/source_style"
+#define GPM_SOURCE_STYLE_SCHEME		GPM_SOURCE_STYLE_DIR "/scheme"
+
 /* UI */
 #define GPM_TOOLBAR_DIR			GPM_PREFS_DIR "/ui/toolbar"
 #define GPM_TOOLBAR_VISIBLE	 	GPM_TOOLBAR_DIR "/toolbar_visible"
@@ -132,6 +136,9 @@
 #define GPM_BOTTOM_PANEL_SIZE		GPM_WINDOW_DIR "/bottom_panel_size"
 #define GPM_BOTTOM_PANEL_ACTIVE_PAGE	GPM_WINDOW_DIR "/bottom_panel_active_page"
 
+#define GPM_FILE_FILTER			"/gedit-2/filefilter"
+#define GPM_ACTIVE_FILE_FILTER		GPM_FILE_FILTER "/id"
+
 /* Encodings */
 #define GPM_ENCODINGS_DIR		GPM_PREFS_DIR "/encodings"
 #define GPM_AUTO_DETECTED_ENCODINGS	GPM_ENCODINGS_DIR "/auto_detected"
@@ -230,6 +237,8 @@
 
 #define GPM_DEFAULT_SEARCH_HIGHLIGHTING_ENABLE 1 /* TRUE */
 
+#define GPM_DEFAULT_SOURCE_STYLE_SCHEME "gvim"
+
 typedef enum {
 	GEDIT_TOOLBAR_SYSTEM = 0,
 	GEDIT_TOOLBAR_ICONS,
@@ -444,6 +453,11 @@ gboolean 		 gedit_prefs_manager_get_enable_search_highlighting (void);
 void			 gedit_prefs_manager_set_enable_search_highlighting (gboolean esh);
 gboolean		 gedit_prefs_manager_enable_search_highlighting_can_set (void);
 
+/* Style scheme */
+gchar			*gedit_prefs_manager_get_source_style_scheme	(void);
+void			 gedit_prefs_manager_set_source_style_scheme	(const gchar *scheme);
+gboolean		 gedit_prefs_manager_source_style_scheme_can_set(void);
+
 /* Global lockdown */
 GeditLockdownMask	 gedit_prefs_manager_get_lockdown			(void);
 
