@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.c,v 1.45 2006/12/02 16:24:02 pborelli Exp $
+ * $Id: gedit-window.c 5646 2007-06-24 19:56:01Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,8 +37,6 @@
 #include <string.h>
 
 #include <glib/gi18n.h>
-#include <gtksourceview/gtksourcelanguage.h>
-#include <gtksourceview/gtksourcelanguagesmanager.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "gedit-ui.h"
@@ -50,7 +48,7 @@
 #include "gedit-utils.h"
 #include "gedit-commands.h"
 #include "gedit-debug.h"
-#include "gedit-languages-manager.h"
+#include "gedit-language-manager.h"
 #include "gedit-prefs-manager-app.h"
 #include "gedit-panel.h"
 #include "gedit-documents-panel.h"
@@ -578,6 +576,69 @@ update_next_prev_doc_sensitivity_per_window (GeditWindow *window)
 	
 }
 
+static void
+received_clipboard_contents (GtkClipboard     *clipboard,
+			     GtkSelectionData *selection_data,
+			     GeditWindow      *window)
+{
+	gboolean sens;
+	GtkAction *action;
+
+	/* getting clipboard contents is async, so we need to
+	 * get the current tab and its state */
+
+	if (window->priv->active_tab != NULL)
+	{
+		GeditTabState state;
+		gboolean state_normal;
+
+		state = gedit_tab_get_state (window->priv->active_tab);
+		state_normal = (state == GEDIT_TAB_STATE_NORMAL);
+
+		sens = state_normal &&
+		       gtk_selection_data_targets_include_text (selection_data);
+	}
+	else
+	{
+		sens = FALSE;
+	}
+
+	action = gtk_action_group_get_action (window->priv->action_group,
+					      "EditPaste");
+
+	gtk_action_set_sensitive (action, sens);
+
+	g_object_unref (window);
+}
+
+static void
+set_paste_sensitivity_according_to_clipboard (GeditWindow  *window,
+					      GtkClipboard *clipboard)
+{
+	GdkDisplay *display;
+
+	display = gtk_clipboard_get_display (clipboard);
+
+	if (gdk_display_supports_selection_notification (display))
+	{
+		gtk_clipboard_request_contents (clipboard,
+						gdk_atom_intern ("TARGETS", FALSE),
+						(GtkClipboardReceivedFunc) received_clipboard_contents,
+						g_object_ref (window));
+	}
+	else
+	{
+		GtkAction *action;
+
+		action = gtk_action_group_get_action (window->priv->action_group,
+						      "EditPaste");
+
+		/* XFIXES extension not availbale, make
+		 * Paste always sensitive */
+		gtk_action_set_sensitive (action, TRUE);
+	}
+}
+
 static void
 set_sensitivity_according_to_tab (GeditWindow *window,
 				  GeditTab    *tab)
@@ -588,6 +649,7 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 	gboolean       b;
 	gboolean       state_normal;
 	GeditTabState  state;
+	GtkClipboard  *clipboard;
 	GeditLockdownMask lockdown;
 
 	g_return_if_fail (GEDIT_TAB (tab));
@@ -602,10 +664,14 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 	view = gedit_tab_get_view (tab);
 	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 
+	clipboard = gtk_widget_get_clipboard (GTK_WIDGET (window),
+					      GDK_SELECTION_CLIPBOARD);
+
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "FileSave");
 	gtk_action_set_sensitive (action,
 				  (state_normal ||
+				   (state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION) ||
 				   (state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)) &&
 				  !gedit_document_get_readonly (doc) &&
 				  !(lockdown & GEDIT_LOCKDOWN_SAVE_TO_DISK));
@@ -614,14 +680,16 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 					      "FileSaveAs");
 	gtk_action_set_sensitive (action,
 				  (state_normal ||
-				  (state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)) &&
+				   (state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION) ||
+				   (state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)) &&
 				  !(lockdown & GEDIT_LOCKDOWN_SAVE_TO_DISK));
-				  	
+
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "FileRevert");
-	gtk_action_set_sensitive (action, 
-				  !gedit_document_is_untitled (doc) &&
-				  state_normal);
+	gtk_action_set_sensitive (action,
+				  (state_normal ||
+				   (state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION)) &&
+				  !gedit_document_is_untitled (doc));
 
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "FilePrintPreview");
@@ -671,8 +739,15 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 				  
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "EditPaste");
-	gtk_action_set_sensitive (action,
-				  state_normal);
+	if (state_normal)
+	{
+		set_paste_sensitivity_according_to_clipboard (window,
+							      clipboard);
+	}
+	else
+	{
+		gtk_action_set_sensitive (action, FALSE);
+	}
 
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "EditDelete");
@@ -749,17 +824,17 @@ language_toggled (GtkToggleAction *action,
 	}
 	else
 	{
-		languages = gedit_languages_manager_get_available_languages_sorted (
-						gedit_get_languages_manager ());
+		languages = gedit_language_manager_get_available_languages_sorted (
+						gedit_get_language_manager ());
 
-		lang = GTK_SOURCE_LANGUAGE (g_slist_nth_data ((GSList *) languages, n));
+		lang = g_slist_nth_data ((GSList *) languages, n);
 	}
 
 	gedit_document_set_language (doc, (GtkSourceLanguage *) lang);
 }
 
 static gchar *
-escape_section_name (gchar *name)
+escape_section_name (const gchar *name)
 {
 	gchar *tmp;
 	gchar *ret;
@@ -786,9 +861,9 @@ create_language_menu_item (GtkSourceLanguage *lang,
 	GtkRadioAction *action;
 	GtkAction *normal_action;
 	GSList *group;
-	gchar *section;
+	const gchar *section;
 	gchar *escaped_section;
-	gchar *lang_name;
+	const gchar *lang_name;
 	gchar *escaped_lang_name;
 	gchar *tip;
 	gchar *path;
@@ -821,7 +896,6 @@ create_language_menu_item (GtkSourceLanguage *lang,
 
 	/* now add the language item to the section */
 	lang_name = gtk_source_language_get_name (lang);
-
 	escaped_lang_name = g_markup_escape_text (lang_name, -1);
 
 	tip = g_strdup_printf (_("Use %s highlight mode"), lang_name);
@@ -834,8 +908,10 @@ create_language_menu_item (GtkSourceLanguage *lang,
 				       NULL,
 				       index);
 
-	gtk_action_group_add_action (window->priv->languages_action_group,
-				     GTK_ACTION (action));
+	/* Action is added with a NULL accel to make the accel overridable */
+	gtk_action_group_add_action_with_accel (window->priv->languages_action_group,
+						GTK_ACTION (action),
+						NULL);
 	g_object_unref (action);
 
 	/* add the action to the same radio group of the "Normal" action */
@@ -858,9 +934,7 @@ create_language_menu_item (GtkSourceLanguage *lang,
 
 	g_free (path);
 	g_free (tip);
-	g_free (lang_name);
 	g_free (escaped_lang_name);
-	g_free (section);
 	g_free (escaped_section);
 }
 
@@ -905,11 +979,11 @@ create_languages_menu (GeditWindow *window)
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action_none), TRUE);
 
 	/* now add all the known languages */
-	languages = gedit_languages_manager_get_available_languages_sorted (
-						gedit_get_languages_manager ());
+	languages = gedit_language_manager_get_available_languages_sorted (
+						gedit_get_language_manager ());
 
 	for (l = languages, i = 0; l != NULL; l = l->next, ++i)
-		create_language_menu_item (GTK_SOURCE_LANGUAGE (l->data),
+		create_language_menu_item (l->data,
 					    i,
 					    id,
 					    window);
@@ -923,7 +997,7 @@ update_languages_menu (GeditWindow *window)
 	GList *l;
 	GtkAction *action;
 	GtkSourceLanguage *lang;
-	gchar *lang_name;
+	const gchar *lang_name;
 	gchar *escaped_lang_name;
 
 	doc = gedit_window_get_active_document (window);
@@ -934,7 +1008,7 @@ update_languages_menu (GeditWindow *window)
 	if (lang != NULL)
 		lang_name = gtk_source_language_get_name (lang);
 	else
-		lang_name = g_strdup ("LangNone");
+		lang_name = "LangNone";
 
 	escaped_lang_name = g_markup_escape_text (lang_name, -1);
 
@@ -961,7 +1035,6 @@ update_languages_menu (GeditWindow *window)
 	}
 
 	g_list_free (actions);
-	g_free (lang_name);
 	g_free (escaped_lang_name);
 }
 
@@ -2352,32 +2425,38 @@ can_search_again (GeditDocument *doc,
 
 static void
 can_undo (GeditDocument *doc,
-          gboolean       can,
-          GeditWindow   *window)
+	  GParamSpec    *pspec,
+	  GeditWindow   *window)
 {
 	GtkAction *action;
-	
+	gboolean sensitive;
+
+	sensitive = gtk_source_buffer_can_undo (GTK_SOURCE_BUFFER (doc));
+
 	if (doc != gedit_window_get_active_document (window))
 		return;
-		
+
 	action = gtk_action_group_get_action (window->priv->action_group,
 					     "EditUndo");
-	gtk_action_set_sensitive (action, can);
+	gtk_action_set_sensitive (action, sensitive);
 }
 
 static void
 can_redo (GeditDocument *doc,
-          gboolean       can,
-          GeditWindow   *window)
+	  GParamSpec    *pspec,
+	  GeditWindow   *window)
 {
 	GtkAction *action;
+	gboolean sensitive;
+
+	sensitive = gtk_source_buffer_can_redo (GTK_SOURCE_BUFFER (doc));
 
 	if (doc != gedit_window_get_active_document (window))
 		return;
-	
+
 	action = gtk_action_group_get_action (window->priv->action_group,
 					     "EditRedo");
-	gtk_action_set_sensitive (action, can);
+	gtk_action_set_sensitive (action, sensitive);
 }
 
 static void
@@ -2554,11 +2633,11 @@ notebook_tab_added (GeditNotebook *notebook,
 			  G_CALLBACK (can_search_again),
 			  window);
 	g_signal_connect (doc,
-			  "can-undo",
+			  "notify::can-undo",
 			  G_CALLBACK (can_undo),
 			  window);
 	g_signal_connect (doc,
-			  "can-redo",
+			  "notify::can-redo",
 			  G_CALLBACK (can_redo),
 			  window);
 	g_signal_connect (doc,
@@ -3093,6 +3172,44 @@ init_panels_visibility (GeditWindow *window)
 			  window);
 }
 
+static void
+clipboard_owner_change (GtkClipboard        *clipboard,
+			GdkEventOwnerChange *event,
+			GeditWindow         *window)
+{
+	set_paste_sensitivity_according_to_clipboard (window,
+						      clipboard);
+}
+
+static void
+window_realized (GtkWidget *window,
+		 gpointer  *data)
+{
+	GtkClipboard *clipboard;
+
+	clipboard = gtk_widget_get_clipboard (window,
+					      GDK_SELECTION_CLIPBOARD);
+
+	g_signal_connect (clipboard,
+			  "owner_change",
+			  G_CALLBACK (clipboard_owner_change),
+			  window);
+}
+
+static void
+window_unrealized (GtkWidget *window,
+		   gpointer  *data)
+{
+	GtkClipboard *clipboard;
+
+	clipboard = gtk_widget_get_clipboard (window,
+					      GDK_SELECTION_CLIPBOARD);
+
+	g_signal_handlers_disconnect_by_func (clipboard,
+					      G_CALLBACK (clipboard_owner_change),
+					      window);
+}
+
 static void
 gedit_window_init (GeditWindow *window)
 {
@@ -3220,6 +3337,17 @@ gedit_window_init (GeditWindow *window)
 	                  G_CALLBACK (drag_data_received_cb), 
 	                  NULL);
 
+	/* we can get the clipboard only after the widget
+	 * is realized */
+	g_signal_connect (window,
+			  "realize",
+			  G_CALLBACK (window_realized),
+			  NULL);
+	g_signal_connect (window,
+			  "unrealize",
+			  G_CALLBACK (window_unrealized),
+			  NULL);
+
 	gedit_debug_message (DEBUG_WINDOW, "Update plugins ui");
 	gedit_plugins_engine_update_plugins_ui (window, TRUE);
 
